@@ -739,7 +739,20 @@ bp_ieee80211_get_stats64(struct net_device *dev,
 }
 #endif
 
+#if LINUX_VERSION_IS_LESS(4,10,0)
+static int __change_mtu(struct net_device *ndev, int new_mtu){
+	if (new_mtu < 0 || new_mtu > 0)
+		return -EINVAL;
+	ndev->mtu = new_mtu;
+	return 0;
+}
+#endif
+
 static const struct net_device_ops ieee80211_dataif_ops = {
+#if LINUX_VERSION_IS_LESS(4,10,0)
+	.ndo_change_mtu = __change_mtu,
+#endif
+
 	.ndo_open		= ieee80211_open,
 	.ndo_stop		= ieee80211_stop,
 	.ndo_uninit		= ieee80211_uninit,
@@ -804,6 +817,10 @@ static u16 ieee80211_monitor_select_queue(struct net_device *dev,
 }
 
 static const struct net_device_ops ieee80211_monitorif_ops = {
+#if LINUX_VERSION_IS_LESS(4,10,0)
+	.ndo_change_mtu = __change_mtu,
+#endif
+
 	.ndo_open		= ieee80211_open,
 	.ndo_stop		= ieee80211_stop,
 	.ndo_uninit		= ieee80211_uninit,
@@ -820,6 +837,10 @@ static const struct net_device_ops ieee80211_monitorif_ops = {
 };
 
 static const struct net_device_ops ieee80211_dataif_8023_ops = {
+#if LINUX_VERSION_IS_LESS(4,10,0)
+	.ndo_change_mtu = __change_mtu,
+#endif
+
 	.ndo_open		= ieee80211_open,
 	.ndo_stop		= ieee80211_stop,
 	.ndo_uninit		= ieee80211_uninit,
@@ -2077,13 +2098,27 @@ int ieee80211_if_add(struct ieee80211_local *local, const char *name,
 
 		netdev_set_default_ethtool_ops(ndev, &ieee80211_ethtool_ops);
 
-		/* MTU range: 256 - 2304 */
+		/* MTU range is normally 256 - 2304, where the upper limit is
+		 * the maximum MSDU size. Monitor interfaces send and receive
+		 * MPDU and A-MSDU frames which may be much larger so we do
+		 * not impose an upper limit in that case.
+		 */
 #if LINUX_VERSION_IS_GEQ(4,10,0)
 		ndev->min_mtu = 256;
 #endif
+		if (type == NL80211_IFTYPE_MONITOR) {
 #if LINUX_VERSION_IS_GEQ(4,10,0)
-		ndev->max_mtu = local->hw.max_mtu;
+			ndev->min_mtu = 0;
 #endif
+#if LINUX_VERSION_IS_GEQ(4,10,0)
+			ndev->max_mtu = 0;
+#endif
+		}
+		else {
+#if LINUX_VERSION_IS_GEQ(4,10,0)
+			ndev->max_mtu = local->hw.max_mtu;
+#endif
+		}
 
 		ret = register_netdevice(ndev);
 		if (ret) {
